Use Google Sheets as a data source for your webapps.

1. Create a spreadsheet
2. Create a webapp that points at that spreadsheet
3. Update the spreadsheet, watch your app update automatically

Why write a CMS for small projects when you could just edit a spreadsheet?

Works in node and the browser.  The ES5 browser build is ~1040 bytes minified/gzipped.

# Relationship to Tabletop.js

This library owes a lot to @jsoma and [Tabletop.js](https://github.com/jsoma/tabletop).

Tabletop's implementation pointed me at the endpoints to hit, and saved me a lot of investigation into the specifics of how to fetch the data from Google Sheets.

Sheetsy is much simpler than Tabletop, by virtue of doing less, avoiding legacy IE and HTTP support, and being composed of pure functions, which allows this library to be more thoroughly tested.

# How to set up your Google Sheet

1. Create a [Google Sheet](https://docs.google.com/spreadsheets/)
2. Navigate to the "File" menu
3. Click "Publish to the Web"
	- It should default to "Entire Document" + "Web page", which is what you want.
	- The defaults should be fine for "Published content & settings" as well - for maximum easiness, you'll want to "Automatically republish when changes are made".
5. Click "Publish"
6. Copy the URL

# Using a published Google Sheet as a data source

This is what you'll do:

1. Turn the URL into a key
2. Using the key, fetch the list of sheets
3. With the original key and a sheet id from the list, fetch a sheet's data

# HTTPS

This library will only work in the browser if your site is served over HTTPS.

Your site should be served over HTTPS anyway.  For local testing, use [ngrok](https://ngrok.com/) or something.

Caveat: if you want to try to work around this, you can implement your own HTTP GET function.  [See below](#optional-argument-getfunction)

# API

```js
const sheetsy = require('sheetsy')
const { urlToKey, getSheetList, getSheet } = sheetsy
```

## `key = urlToKey(url)`

Given a `url` string, returns a `key` string.  Throws an error if no key is found.

```js
const key = urlToKey(
	'https://docs.google.com/spreadsheets/d/14uk6kljx-tpGJeObmi22DkAyVRFK5Z1qKmSXy1ewuHs/pubhtml'
) // => '14uk6kljx-tpGJeObmi22DkAyVRFK5Z1qKmSXy1ewuHs'
```

Tabletop.js notes that [some publish URLs don't work for some reason](https://github.com/jsoma/tabletop#if-your-publish-to-web-url-doesnt-work) - if you run into a published document that doesn't work for some reason, please open an issue with the link that you're using and a description of the error, I'd be interested to create some tests for those cases.

## `promise = getSheetList(key, [getFunction])`

Given a `key` string, returns a Promise that resolves to an array of objects describing the sheets in the document.

```js
getSheetList('14uk6kljx-tpGJeObmi22DkAyVRFK5Z1qKmSXy1ewuHs').then(sheets => {
	sheets // => [
	//		{ name: 'Herp', id: 'od6' },
	//		{ name: 'Derp', id: 'of6b9b5' }
	//	]
})
```

## `promise = getSheet(key, id, [getFunction])`

Given a `key` string and an `id` string from the sheet list results, returns a Promise that resolves to an array of rows from the sheet.

```js
getSheet('14uk6kljx-tpGJeObmi22DkAyVRFK5Z1qKmSXy1ewuHs', '0d6').then(sheet => {
	sheet // => [
	//		[ `Something's up`, `That's "cool"` ],
	//		[ `a3`, `b3`, `c3` ]
	//	]
})
```

In addition, you can also access values of a row by property name.  In this example, the value in cell A1 is "First header":

```js
getSheet('14uk6kljx-tpGJeObmi22DkAyVRFK5Z1qKmSXy1ewuHs', '0d6').then(sheet => {
	const firstRow = sheet[0]
	firstRow.firstheader // => `Something's up`
	firstRow.secondheader // => `That's "cool"`
})
```
### Column names/property names

The column names are generated by Google Sheets, not by this library.

Whether you format the first row as a header or not, Google Sheets will treat it as if it is the header row of your sheet, and will use the values in those columns to generate the names of all of the column values in all the rows following.

It appears to strip spaces and lowercase your text.

## Optional argument: `getFunction`

The `getSheet` and `getSheetList` functions take an optional getter function.  This is automatically given a function backed by `XMLHttpRequest` in the browser, or the [`got`](https://github.com/sindresorhus/got) module in node.

You can pass in your own function if you want to try to get it working from an HTTP site or something.  The function is expected to take the url as a string, make a GET request to that url, and return the body of the response as parsed JSON.

# How to use in the browser or node

Your browser bundler will need to respect the `browser` field in `package.json`.  Given that, all you need to do is `import sheetsy from 'sheetsy'` or `const sheetsy = require('sheetsy')`.

# Possible future features

If you find yourself needing these, open an issue with full details about your use case, or a pull request with tests.

- Return some other metadata with the document/sheets, such as author or last updated time
- Handle private sheets

# License

[WTFPL](http://wtfpl2.com)
